@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.3 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2020, Mark Calabretta
+  WCSLIB 8.5 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2025, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -17,14 +17,12 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: dis.h,v 7.3.1.2 2020/08/17 12:10:44 mcalabre Exp mcalabre $
+  $Id: dis.h,v 8.5 2025/12/06 13:47:41 mcalabre Exp $
 *=============================================================================
 *
-* WCSLIB 7.3 - C routines that implement the FITS World Coordinate System
+* WCSLIB 8.5 - C routines that implement the FITS World Coordinate System
 * (WCS) standard.  Refer to the README file provided with WCSLIB for an
 * overview of the library.
 *
@@ -91,6 +89,10 @@
 * WCSLIB would handle a FITS header containing these keywords, along with
 * CQDISia = 'TPV' and the required DQia.NAXES and DQia.AXIS.ihat keywords.
 *
+* Note that, as defined, TPV assumes that CDi_ja is used to define the linear
+* transformation.  The section on historical idiosyncrasies (below) cautions
+* about translating CDi_ja to PCi_ja plus CDELTia in this case.
+*
 * SIP - Simple Imaging Polynomial:
 * --------------------------------
 * These routines also support the Simple Imaging Polynomial (SIP), whose
@@ -165,6 +167,10 @@
 * the ZPN projection parameters are also encoded in WATi_n, and wcspih()
 * translates these to standard PVi_ma.
 *
+* Note that, as defined, TNX and ZPX assume that CDi_ja is used to define the
+* linear transformation.  The section on historical idiosyncrasies (below)
+* cautions about translating CDi_ja to PCi_ja plus CDELTia in this case.
+*
 * TPD - Template Polynomial Distortion:
 * -------------------------------------
 * The "Template Polynomial Distortion" (TPD) is a superset of the TPV, SIP,
@@ -245,6 +251,12 @@
 * where r = sqrt(xx + yy).  Note that even powers of r are excluded since they
 * can be accomodated by powers of (xx + yy).
 *
+* Note here that "x" refers to the axis to which the distortion function is
+* attached, with "y" being the complementary axis.  So, for example, with
+* longitude on axis 1 and latitude on axis 2, for TPD attached to axis 1, "x"
+* refers to axis 1 and "y" to axis 2.  For TPD attached to axis 2, "x" refers
+* to axis 2, and "y" to axis 1.
+*
 * TPV uses all terms up to 39.  The m in its PVi_ma keywords translates
 * directly to the TPD coefficient number.
 *
@@ -306,6 +318,13 @@
 *
 * where the value corresponds to CRPIXja.
 *
+* Likewise, because TPV, TNX, and ZPX are defined in terms of CDi_ja, the
+* independent variables of the polynomial are intermediate world coordinates
+* rather than intermediate pixel coordinates.  Because sequent distortions
+* are always applied before CDELTia, if CDi_ja is translated to PCi_ja plus
+* CDELTia, then either CDELTia must be unity, or the distortion polynomial
+* coefficients must be adjusted to account for the change of scale.
+*
 * Summary of the dis routines:
 * ----------------------------
 * These routines apply the distortion functions defined by the extension to
@@ -318,7 +337,9 @@
 * dpfill(), dpkeyi(), and dpkeyd() are provided to manage the dpkey struct.
 *
 * disndp(), disini(), disinit(), discpy(), and disfree() are provided to
-* manage the disprm struct, and another, disprt(), prints its contents.
+* manage the disprm struct, dissize() computes its total size including
+* allocated memory, disenq() returns information about the state of the
+* struct, and disprt() prints its contents.
 *
 * disperr() prints the error message(s) (if any) stored in a disprm struct.
 *
@@ -534,7 +555,7 @@
 * PLEASE NOTE: disfree() must not be invoked on a disprm struct that was not
 * initialized by disinit().
 *
-* Given:
+* Given and returned:
 *   dis       struct disprm*
 *                       Distortion function parameters.
 *
@@ -544,6 +565,60 @@
 *                         1: Null disprm pointer passed.
 *
 *
+* dissize() - Compute the size of a disprm struct
+* -----------------------------------------------
+* dissize() computes the full size of a disprm struct, including allocated
+* memory.
+*
+* Given:
+*   dis       const struct disprm*
+*                       Distortion function parameters.
+*
+*                       If NULL, the base size of the struct and the allocated
+*                       size are both set to zero.
+*
+* Returned:
+*   sizes     int[2]    The first element is the base size of the struct as
+*                       returned by sizeof(struct disprm).  The second element
+*                       is the total allocated size, in bytes, assuming that
+*                       the allocation was done by disini().  This figure
+*                       includes memory allocated for members of constituent
+*                       structs, such as disprm::dp.
+*
+*                       It is not an error for the struct not to have been set
+*                       up via tabset(), which normally results in additional
+*                       memory allocation.
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Success.
+*
+*
+* disenq() - enquire about the state of a disprm struct
+* -----------------------------------------------------
+* disenq() may be used to obtain information about the state of a disprm
+* struct.  The function returns a true/false answer for the enquiry asked.
+*
+* Given:
+*   dis       const struct disprm*
+*                       Distortion function parameters.
+*
+*   enquiry   int       Enquiry according to the following parameters:
+*                         DISENQ_MEM: memory in the struct is being managed by
+*                                     WCSLIB (see disinit()).
+*                         DISENQ_SET: the struct has been set up by disset().
+*                         DISENQ_BYP: the struct is in bypass mode (see
+*                                     disset()).
+*                       These may be combined by logical OR, e.g.
+*                       DISENQ_MEM | DISENQ_SET.  The enquiry result will be
+*                       the logical AND of the individual results.
+*
+* Function return value:
+*             int       Enquiry result:
+*                         0: No.
+*                         1: Yes.
+*
+*
 * disprt() - Print routine for the disprm struct
 * ----------------------------------------------
 * disprt() prints the contents of a disprm struct using wcsprintf().  Mainly
@@ -604,6 +679,13 @@
 * disp2x() and disx2p() if the disprm::flag is anything other than a
 * predefined magic value.
 *
+* disset() normally operates regardless of the value of disprm::flag; i.e.
+* even if a struct was previously set up it will be reset unconditionally.
+* However, a disprm struct may be put into "bypass" mode by invoking disset()
+* initially with disprm::flag == 1 (rather than 0).  disset() will return
+* immediately if invoked on a struct in that state.  To take a struct out of
+* bypass mode, simply reset disprm::flag to zero.  See also disenq().
+*
 * Given and returned:
 *   dis       struct disprm*
 *                       Distortion function parameters.
@@ -805,8 +887,8 @@
 * the user.
 *
 *   int flag
-*     (Given and returned) This flag must be set to zero whenever any of the
-*     following members of the disprm struct are set or modified:
+*     (Given and returned) This flag must be set to zero (or 1, see disset())
+*     whenever any of the following disprm members are set or changed:
 *
 *       - disprm::naxis,
 *       - disprm::dtype,
@@ -943,8 +1025,6 @@
 *     (For internal use only.)
 *   int (**disx2p)(DISX2P_ARGS)
 *     (For internal use only.)
-*   double *tmpmem
-*     (For internal use only.)
 *   int m_flag
 *     (For internal use only.)
 *   int m_naxis
@@ -1005,6 +1085,11 @@
 extern "C" {
 #endif
 
+enum disenq_enum {
+  DISENQ_MEM = 1,		// disprm struct memory is managed by WCSLIB.
+  DISENQ_SET = 2,		// disprm struct has been set up.
+  DISENQ_BYP = 4,		// disprm struct is in bypass mode.
+};
 
 extern const char *dis_errmsg[];
 
@@ -1054,8 +1139,8 @@ struct disprm {
   int    ndp;			// Number of DPja or DQia keywords, and the
   int    ndpmax;		// number for which space was allocated.
   struct dpkey *dp;		// DPja or DQia keyvalues (not both).
-  double *maxdis;		// For each axis, the maximum distortion.
   double totdis;		// The maximum combined distortion.
+  double *maxdis;		// For each axis, the maximum distortion.
 
   // Information derived from the parameters supplied.
   //--------------------------------------------------------------------------
@@ -1073,17 +1158,16 @@ struct disprm {
   int    i_naxis;		// Dimension of the internal arrays.
   int    ndis;			// The number of distortion functions.
 
-  // Error handling, if enabled.
+  // Error messaging, if enabled.
   //--------------------------------------------------------------------------
   struct wcserr *err;
 
+  //--------------------------------------------------------------------------
   // Private - the remainder are for internal use.
   //--------------------------------------------------------------------------
   int (**disp2x)(DISP2X_ARGS);	// For each axis, pointers to the
   int (**disx2p)(DISX2P_ARGS);	// distortion function and its inverse.
 
-  double *tmpmem;
-
   int    m_flag, m_naxis;	// The remainder are for memory management.
   char   (*m_dtype)[72];
   struct dpkey *m_dp;
@@ -1111,6 +1195,10 @@ int discpy(int alloc, const struct disprm *dissrc, struct disprm *disdst);
 
 int disfree(struct disprm *dis);
 
+int dissize(const struct disprm *dis, int sizes[2]);
+
+int disenq(const struct disprm *dis, int enquiry);
+
 int disprt(const struct disprm *dis);
 
 int disperr(const struct disprm *dis, const char *prefix);
