@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.3 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2020, Mark Calabretta
+  WCSLIB 8.5 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2025, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -17,14 +17,12 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: wcs.h,v 7.3.1.2 2020/08/17 12:10:44 mcalabre Exp mcalabre $
+  $Id: wcs.h,v 8.5 2025/12/06 13:47:41 mcalabre Exp $
 *=============================================================================
 *
-* WCSLIB 7.3 - C routines that implement the FITS World Coordinate System
+* WCSLIB 8.5 - C routines that implement the FITS World Coordinate System
 * (WCS) standard.  Refer to the README file provided with WCSLIB for an
 * overview of the library.
 *
@@ -66,10 +64,10 @@
 * set by the user, and others that are maintained by these routines, somewhat
 * like a C++ class but with no encapsulation.
 *
-* wcsnpv(), wcsnps(), wcsini(), wcsinit(), wcssub(), and wcsfree() are
-* provided to manage the wcsprm struct and another, wcsprt(), prints its
-* contents.  Refer to the description of the wcsprm struct for an explanation
-* of the anticipated usage of these routines.  wcscopy(), which does a deep
+* wcsnpv(), wcsnps(), wcsini(), wcsinit(), wcssub(), wcsfree(), and wcstrim(),
+* are provided to manage the wcsprm struct, wcssize() computes its total size
+* including allocated memory, wcsenq() returns information about the state of
+* the struct, and wcsprt() prints its contents.  wcscopy(), which does a deep
 * copy of one wcsprm struct to another, is defined as a preprocessor macro
 * function that invokes wcssub().
 *
@@ -90,8 +88,10 @@
 * pixel coordinate a hybrid routine, wcsmix(), iteratively solves for the
 * unknown elements.
 *
-* wcssptr() translates the spectral axis in a wcsprm struct.  For example, a
-* 'FREQ' axis may be translated into 'ZOPT-F2W' and vice versa.
+* wcsccs() changes the celestial coordinate system of a wcsprm struct, for
+* example, from equatorial to galactic, and wcssptr() translates the spectral
+* axis.  For example, a 'FREQ' axis may be translated into 'ZOPT-F2W' and vice
+* versa.
 *
 * wcslib_version() returns the WCSLIB version number.
 *
@@ -326,6 +326,7 @@
 *                         WCSSUB_CUBEFACE:  Quadcube CUBEFACE axis.
 *                         WCSSUB_SPECTRAL:  Spectral axis.
 *                         WCSSUB_STOKES:    Stokes axis.
+*                         WCSSUB_TIME:      Time axis.
 *
 *                       Refer to the notes (below) for further usage examples.
 *
@@ -357,38 +358,39 @@
 *                       wcsprm::err if enabled, see wcserr_enable().
 *
 * Notes:
-*   Combinations of subimage axes of particular types may be extracted in the
-*   same order as they occur in the input image by combining preprocessor
-*   codes, for example
+*   1: Combinations of subimage axes of particular types may be extracted in
+*      the same order as they occur in the input image by combining
+*      preprocessor codes, for example
 *
-=     *nsub = 1;
-=     axes[0] = WCSSUB_LONGITUDE | WCSSUB_LATITUDE | WCSSUB_SPECTRAL;
+=        *nsub = 1;
+=        axes[0] = WCSSUB_LONGITUDE | WCSSUB_LATITUDE | WCSSUB_SPECTRAL;
 *
-*   would extract the longitude, latitude, and spectral axes in the same order
-*   as the input image.  If one of each were present, *nsub = 3 would be
-*   returned.
+*      would extract the longitude, latitude, and spectral axes in the same
+*      order as the input image.  If one of each were present, *nsub = 3 would
+*      be returned.
 *
-*   For convenience, WCSSUB_CELESTIAL is defined as the combination
-*   WCSSUB_LONGITUDE | WCSSUB_LATITUDE | WCSSUB_CUBEFACE.
+*      For convenience, WCSSUB_CELESTIAL is defined as the combination
+*      WCSSUB_LONGITUDE | WCSSUB_LATITUDE | WCSSUB_CUBEFACE.
 *
-*   The codes may also be negated to extract all but the types specified, for
-*   example
+*      The codes may also be negated to extract all but the types specified,
+*      for example
 *
-=     *nsub = 4;
-=     axes[0] = WCSSUB_LONGITUDE;
-=     axes[1] = WCSSUB_LATITUDE;
-=     axes[2] = WCSSUB_CUBEFACE;
-=     axes[3] = -(WCSSUB_SPECTRAL | WCSSUB_STOKES);
+=        *nsub = 4;
+=        axes[0] = WCSSUB_LONGITUDE;
+=        axes[1] = WCSSUB_LATITUDE;
+=        axes[2] = WCSSUB_CUBEFACE;
+=        axes[3] = -(WCSSUB_SPECTRAL | WCSSUB_STOKES);
 *
-*   The last of these specifies all axis types other than spectral or Stokes.
-*   Extraction is done in the order specified by axes[] a longitude axis (if
-*   present) would be extracted first (via axes[0]) and not subsequently (via
-*   axes[3]).  Likewise for the latitude and cubeface axes in this example.
+*      The last of these specifies all axis types other than spectral or
+*      Stokes.  Extraction is done in the order specified by axes[] a
+*      longitude axis (if present) would be extracted first (via axes[0]) and
+*      not subsequently (via axes[3]).  Likewise for the latitude and cubeface
+*      axes in this example.
 *
-*   From the foregoing, it is apparent that the value of *nsub returned may be
-*   less than or greater than that given.  However, it will never exceed the
-*   number of axes in the input image (plus the number of newly-created axes
-*   if any were specified on input).
+*      From the foregoing, it is apparent that the value of *nsub returned may
+*      be less than or greater than that given.  However, it will never exceed
+*      the number of axes in the input image (plus the number of newly-created
+*      axes if any were specified on input).
 *
 *
 * wcscompare() - Compare two wcsprm structs for equality
@@ -449,7 +451,7 @@
 * PLEASE NOTE: wcsfree() must not be invoked on a wcsprm struct that was not
 * initialized by wcsinit().
 *
-* Returned:
+* Given and returned:
 *   wcs       struct wcsprm*
 *                       Coordinate transformation parameters.
 *
@@ -459,6 +461,120 @@
 *                         1: Null wcsprm pointer passed.
 *
 *
+* wcstrim() - Free unused arrays in the wcsprm struct
+* ---------------------------------------------------
+* wcstrim() frees memory allocated by wcsinit() for arrays in the wcsprm
+* struct that remains unused after it has been set up by wcsset().
+*
+* The free'd array members are associated with FITS WCS keyrecords that are
+* rarely used and usually just bloat the struct: wcsprm::crota, wcsprm::colax,
+* wcsprm::cname, wcsprm::crder, wcsprm::csyer, wcsprm::czphs, and
+* wcsprm::cperi.  If unused, wcsprm::pv, wcsprm::ps, and wcsprm::cd are also
+* freed.
+*
+* Once these arrays have been freed, a test such as
+=
+=        if (!undefined(wcs->cname[i])) {...}
+=
+* must be protected as follows
+=
+=        if (wcs->cname && !undefined(wcs->cname[i])) {...}
+=
+* In addition, if wcsprm::npv is non-zero but less than wcsprm::npvmax, then
+* the unused space in wcsprm::pv will be recovered (using realloc()).
+* Likewise for wcsprm::ps.
+*
+* Given and returned:
+*   wcs       struct wcsprm*
+*                       Coordinate transformation parameters.
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Success.
+*                         1: Null wcsprm pointer passed.
+*                        14: wcsprm struct is unset.
+*
+*
+* wcssize() - Compute the size of a wcsprm struct
+* -----------------------------------------------
+* wcssize() computes the full size of a wcsprm struct, including allocated
+* memory.
+*
+* Given:
+*   wcs       const struct wcsprm*
+*                       Coordinate transformation parameters.
+*
+*                       If NULL, the base size of the struct and the allocated
+*                       size are both set to zero.
+*
+* Returned:
+*   sizes     int[2]    The first element is the base size of the struct as
+*                       returned by sizeof(struct wcsprm).  The second element
+*                       is the total allocated size, in bytes, assuming that
+*                       the allocation was done by wcsini().  This figure
+*                       includes memory allocated for members of constituent
+*                       structs, such as wcsprm::lin.
+*
+*                       It is not an error for the struct not to have been set
+*                       up via wcsset(), which normally results in additional
+*                       memory allocation.
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Success.
+*
+*
+* auxsize() - Compute the size of a auxprm struct
+* -----------------------------------------------
+* auxsize() computes the full size of an auxprm struct, including allocated
+* memory.
+*
+* Given:
+*   aux       const struct auxprm*
+*                       Auxiliary coordinate information.
+*
+*                       If NULL, the base size of the struct and the allocated
+*                       size are both set to zero.
+*
+* Returned:
+*   sizes     int[2]    The first element is the base size of the struct as
+*                       returned by sizeof(struct auxprm).  The second element
+*                       is the total allocated size, in bytes, currently zero.
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Success.
+*
+*
+* wcsenq() - enquire about the state of a wcsprm struct
+* -----------------------------------------------------
+* wcsenq() may be used to obtain information about the state of a wcsprm
+* struct.  The function returns a true/false answer for the enquiry asked.
+*
+* Given:
+*   wcs       const struct wcsprm*
+*                       Coordinate transformation parameters.
+*
+*   enquiry   int       Enquiry according to the following parameters:
+*                         WCSENQ_MEM: memory in the struct is being managed by
+*                                     WCSLIB (see wcsini()).
+*                         WCSENQ_SET: the struct has been set up by wcsset().
+*                         WCSENQ_BYP: the struct is in bypass mode (see
+*                                     wcsset()).
+*                         WCSENQ_CHK: the struct is self-consistent in that
+*                                     no changes have been made to any of the
+*                                     "parameters to be given" since the last
+*                                     call to wcsset().
+*                       These may be combined by logical OR, e.g.
+*                       WCSENQ_MEM | WCSENQ_SET.  The enquiry result will be
+*                       the logical AND of the individual results.
+*
+* Function return value:
+*             int       Enquiry result:
+*                         0: False.
+*                         1: True.
+*
+*
 * wcsprt() - Print routine for the wcsprm struct
 * ----------------------------------------------
 * wcsprt() prints the contents of a wcsprm struct using wcsprintf().  Mainly
@@ -539,6 +655,13 @@
 * wcsp2s() and wcss2p() if the wcsprm::flag is anything other than a
 * predefined magic value.
 *
+* wcsset() normally operates regardless of the value of wcsprm::flag; i.e.
+* even if a struct was previously set up it will be reset unconditionally.
+* However, a wcsprm struct may be put into "bypass" mode by invoking wcsset()
+* initially with wcsprm::flag == 1 (rather than 0).  wcsset() will return
+* immediately if invoked on a struct in that state.  To take a struct out of
+* bypass mode, simply reset wcsprm::flag to zero.  See also wcsenq().
+*
 * Given and returned:
 *   wcs       struct wcsprm*
 *                       Coordinate transformation parameters.
@@ -560,9 +683,9 @@
 *                       wcsprm::err if enabled, see wcserr_enable().
 *
 * Notes:
-*   wcsset() always enables strict bounds checking in the projection routines
-*   (via a call to prjini()).  Use wcsbchk() to modify bounds-checking after
-*   wcsset() is invoked.
+*   1: wcsset() always enables strict bounds checking in the projection
+*      routines (via a call to prjini()).  Use wcsbchk() to modify
+*      bounds-checking after wcsset() is invoked.
 *
 *
 * wcsp2s() - Pixel-to-world transformation
@@ -591,7 +714,9 @@
 *                       imgcrd[][wcs.lat] are the projected x-, and
 *                       y-coordinates in pseudo "degrees".  For spectral
 *                       axes, imgcrd[][wcs.spec] is the intermediate spectral
-*                       coordinate, in SI units.
+*                       coordinate, in SI units.  For time axes,
+*                       imgcrd[][wcs.time] is the intermediate time
+*                       coordinate.
 *
 *   phi,theta double[ncoord]
 *                       Longitude and latitude in the native coordinate system
@@ -600,9 +725,10 @@
 *   world     double[ncoord][nelem]
 *                       Array of world coordinates.  For celestial axes,
 *                       world[][wcs.lng] and world[][wcs.lat] are the
-*                       celestial longitude and latitude [deg].  For
-*                       spectral axes, imgcrd[][wcs.spec] is the intermediate
-*                       spectral coordinate, in SI units.
+*                       celestial longitude and latitude [deg].  For spectral
+*                       axes, world[][wcs.spec] is the spectral coordinate, in
+*                       SI units.  For time axes, world[][wcs.time] is the
+*                       time coordinate.
 *
 *   stat      int[ncoord]
 *                       Status return value for each coordinate:
@@ -650,7 +776,8 @@
 *                       world[][wcs.lng] and world[][wcs.lat] are the
 *                       celestial longitude and latitude [deg]. For spectral
 *                       axes, world[][wcs.spec] is the spectral coordinate, in
-*                       SI units.
+*                       SI units.  For time axes, world[][wcs.time] is the
+*                       time coordinate.
 *
 * Returned:
 *   phi,theta double[ncoord]
@@ -665,7 +792,9 @@
 *                       projections with a CUBEFACE axis the face number is
 *                       also returned in imgcrd[][wcs.cubeface].  For
 *                       spectral axes, imgcrd[][wcs.spec] is the intermediate
-*                       spectral coordinate, in SI units.
+*                       spectral coordinate, in SI units.  For time axes,
+*                       imgcrd[][wcs.time] is the intermediate time
+*                       coordinate.
 *
 *   pixcrd    double[ncoord][nelem]
 *                       Array of pixel coordinates.
@@ -779,31 +908,167 @@
 *                       wcsprm::err if enabled, see wcserr_enable().
 *
 * Notes:
-*   Initially the specified solution interval is checked to see if it's a
-*   "crossing" interval.  If it isn't, a search is made for a crossing
-*   solution by iterating on the unknown celestial coordinate starting at the
-*   upper limit of the solution interval and decrementing by the specified
-*   step size.  A crossing is indicated if the trial value of the pixel
-*   coordinate steps through the value specified.  If a crossing interval is
-*   found then the solution is determined by a modified form of "regula falsi"
-*   division of the crossing interval.  If no crossing interval was found
-*   within the specified solution interval then a search is made for a
-*   "non-crossing" solution as may arise from a point of tangency.  The
-*   process is complicated by having to make allowance for the discontinuities
-*   that occur in all map projections.
+*   1: Initially the specified solution interval is checked to see if it's a
+*      "crossing" interval.  If it isn't, a search is made for a crossing
+*      solution by iterating on the unknown celestial coordinate starting at
+*      the upper limit of the solution interval and decrementing by the
+*      specified step size.  A crossing is indicated if the trial value of the
+*      pixel coordinate steps through the value specified.  If a crossing
+*      interval is found then the solution is determined by a modified form of
+*      "regula falsi" division of the crossing interval.  If no crossing
+*      interval was found within the specified solution interval then a search
+*      is made for a "non-crossing" solution as may arise from a point of
+*      tangency.  The process is complicated by having to make allowance for
+*      the discontinuities that occur in all map projections.
 *
-*   Once one solution has been determined others may be found by subsequent
-*   invokations of wcsmix() with suitably restricted solution intervals.
+*      Once one solution has been determined others may be found by subsequent
+*      invokations of wcsmix() with suitably restricted solution intervals.
 *
-*   Note the circumstance that arises when the solution point lies at a native
-*   pole of a projection in which the pole is represented as a finite curve,
-*   for example the zenithals and conics.  In such cases two or more valid
-*   solutions may exist but wcsmix() only ever returns one.
+*      Note the circumstance that arises when the solution point lies at a
+*      native pole of a projection in which the pole is represented as a
+*      finite curve, for example the zenithals and conics.  In such cases two
+*      or more valid solutions may exist but wcsmix() only ever returns one.
 *
-*   Because of its generality wcsmix() is very compute-intensive.  For
-*   compute-limited applications more efficient special-case solvers could be
-*   written for simple projections, for example non-oblique cylindrical
-*   projections.
+*      Because of its generality wcsmix() is very compute-intensive.  For
+*      compute-limited applications more efficient special-case solvers could
+*      be written for simple projections, for example non-oblique cylindrical
+*      projections.
+*
+*
+* wcsccs() - Change celestial coordinate system
+* ---------------------------------------------
+* wcsccs() changes the celestial coordinate system of a wcsprm struct.  For
+* example, from equatorial to galactic coordinates.
+*
+* Parameters that define the spherical coordinate transformation, essentially
+* being three Euler angles, must be provided.  Thereby wcsccs() does not need
+* prior knowledge of specific celestial coordinate systems.  It also has the
+* advantage of making it completely general.
+*
+* Auxiliary members of the wcsprm struct relating to equatorial celestial
+* coordinate systems may also be changed.
+*
+* Only orthodox spherical coordinate systems are supported.  That is, they
+* must be right-handed, with latitude increasing from zero at the equator to
+* +90 degrees at the pole.  This precludes systems such as aziumuth and zenith
+* distance, which, however, could be handled as negative azimuth and
+* elevation.
+*
+* PLEASE NOTE: Information in the wcsprm struct relating to the original
+* coordinate system will be overwritten and therefore lost.  If this is
+* undesirable, invoke wcsccs() on a copy of the struct made with wcssub().
+* The wcsprm struct is reset on return with an explicit call to wcsset().
+*
+* Given and returned:
+*   wcs       struct wcsprm*
+*                       Coordinate transformation parameters.  Particular
+*                       "values to be given" elements of the wcsprm struct
+*                       are modified.
+*
+* Given:
+*   lng2p1,
+*   lat2p1    double    Longitude and latitude in the new celestial coordinate
+*                       system of the pole (i.e. latitude +90) of the original
+*                       system [deg].  See notes 1 and 2 below.
+*
+*   lng1p2    double    Longitude in the original celestial coordinate system
+*                       of the pole (i.e. latitude +90) of the new system
+*                       [deg].  See note 1 below.
+*
+*   clng,clat const char*
+*                       Longitude and latitude identifiers of the new CTYPEia
+*                       celestial axis codes, without trailing dashes.  For
+*                       example, "RA" and "DEC" or "GLON" and "GLAT".  Up to
+*                       four characters are used, longer strings need not be
+*                       null-terminated.
+*
+*   radesys   const char*
+*                       Used when transforming to equatorial coordinates,
+*                       identified by clng == "RA" and clat = "DEC".  May be
+*                       set to the null pointer to preserve the current value.
+*                       Up to 71 characters are used, longer strings need not
+*                       be null-terminated.
+*
+*                       If the new coordinate system is anything other than
+*                       equatorial, then wcsprm::radesys will be cleared.
+*
+*   equinox   double    Used when transforming to equatorial coordinates.  May
+*                       be set to zero to preserve the current value.
+*
+*                       If the new coordinate system is not equatorial, then
+*                       wcsprm::equinox will be marked as undefined.
+*
+*   alt       const char*
+*                       Character code for alternate coordinate descriptions
+*                       (i.e. the 'a' in keyword names such as CTYPEia).  This
+*                       is blank for the primary coordinate description, or
+*                       one of the 26 upper-case letters, A-Z.  May be set to
+*                       the null pointer, or null string if no change is
+*                       required.
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Success.
+*                         1: Null wcsprm pointer passed.
+*                        12: Invalid subimage specification (no celestial
+*                            axes).
+*
+* Notes:
+*   1: Follows the prescription given in WCS Paper II, Sect. 2.7 for changing
+*      celestial coordinates.
+*
+*      The implementation takes account of indeterminacies that arise in that
+*      prescription in the particular cases where one of the poles of the new
+*      system is at the fiducial point, or one of them is at the native pole.
+*
+*   2: If lat2p1 == +90, i.e. where the poles of the two coordinate systems
+*      coincide, then the spherical coordinate transformation becomes a simple
+*      change in origin of longitude given by
+*      lng2 = lng1 + (lng2p1 - lng1p2 - 180), and lat2 = lat1, where
+*      (lng2,lat2) are coordinates in the new system, and (lng1,lat1) are
+*      coordinates in the original system.
+*
+*      Likewise, if lat2p1 == -90, then lng2 = -lng1 + (lng2p1 + lng1p2), and
+*      lat2 = -lat1.
+*
+*   3: For example, if the original coordinate system is B1950 equatorial and
+*      the desired new coordinate system is galactic, then
+*
+*      - (lng2p1,lat2p1) are the galactic coordinates of the B1950 celestial
+*        pole, defined by the IAU to be (123.0,+27.4), and lng1p2 is the B1950
+*        right ascension of the galactic pole, defined as 192.25.  Clearly
+*        these coordinates are fixed for a particular coordinate
+*        transformation.
+*
+*      - (clng,clat) would be 'GLON' and 'GLAT', these being the FITS standard
+*        identifiers for galactic coordinates.
+*
+*      - Since the new coordinate system is not equatorial, wcsprm::radesys
+*        and wcsprm::equinox will be cleared.
+*
+*   4. The coordinates required for some common transformations (obtained from
+*      https://ned.ipac.caltech.edu/coordinate_calculator) are as follows:
+*
+=      (123.0000,+27.4000) galactic coordinates of B1950 celestial pole,
+=      (192.2500,+27.4000) B1950 equatorial coordinates of galactic pole.
+*
+=      (122.9319,+27.1283) galactic coordinates of J2000 celestial pole,
+=      (192.8595,+27.1283) J2000 equatorial coordinates of galactic pole.
+*
+=      (359.6774,+89.7217) B1950 equatorial coordinates of J2000 pole,
+=      (180.3162,+89.7217) J2000 equatorial coordinates of B1950 pole.
+*
+=      (270.0000,+66.5542) B1950 equatorial coordinates of B1950 ecliptic pole,
+=      ( 90.0000,+66.5542) B1950 ecliptic coordinates of B1950 celestial pole.
+*
+=      (270.0000,+66.5607) J2000 equatorial coordinates of J2000 ecliptic pole,
+=      ( 90.0000,+66.5607) J2000 ecliptic coordinates of J2000 celestial pole.
+*
+=      ( 26.7315,+15.6441) supergalactic coordinates of B1950 celestial pole,
+=      (283.1894,+15.6441) B1950 equatorial coordinates of supergalactic pole.
+*
+=      ( 26.4505,+15.7089) supergalactic coordinates of J2000 celestial pole,
+=      (283.7542,+15.7089) J2000 equatorial coordinates of supergalactic pole.
 *
 *
 * wcssptr() - Spectral axis translation
@@ -811,6 +1076,11 @@
 * wcssptr() translates the spectral axis in a wcsprm struct.  For example, a
 * 'FREQ' axis may be translated into 'ZOPT-F2W' and vice versa.
 *
+* PLEASE NOTE: Information in the wcsprm struct relating to the original
+* coordinate system will be overwritten and therefore lost.  If this is
+* undesirable, invoke wcssptr() on a copy of the struct made with wcssub().
+* The wcsprm struct is reset on return with an explicit call to wcsset().
+*
 * Given and returned:
 *   wcs       struct wcsprm*
 *                       Coordinate transformation parameters.
@@ -900,8 +1170,8 @@
 * string members and null-fills the character array.
 *
 *   int flag
-*     (Given and returned) This flag must be set to zero whenever any of the
-*     following wcsprm struct members are set or changed:
+*     (Given and returned) This flag must be set to zero (or 1, see wcsset())
+*     whenever any of the following wcsprm members are set or changed:
 *
 *       - wcsprm::naxis (q.v., not normally set by the user),
 *       - wcsprm::crpix,
@@ -927,8 +1197,8 @@
 *       - wcsprm::wtb.
 *
 *     This signals the initialization routine, wcsset(), to recompute the
-*     returned members of the celprm struct.  celset() will reset flag to
-*     indicate that this has been done.
+*     returned members of the linprm, celprm, spcprm, and tabprm structs.
+*     wcsset() will reset flag to indicate that this has been done.
 *
 *     PLEASE NOTE: flag should be set to -1 when wcsinit() is called for the
 *     first time for a particular wcsprm struct in order to initialize memory
@@ -1100,38 +1370,45 @@
 *
 *     - Bit 1: CDi_ja is present.
 *
-*       Matrix elements in the IRAF convention are
-*       equivalent to the product CDi_ja = CDELTia * PCi_ja, but the
-*       defaults differ from that of the PCi_ja matrix.  If one or more
-*       CDi_ja keywords are present then all unspecified CDi_ja default to
-*       zero.  If no CDi_ja (or CROTAi) keywords are present, then the
-*       header is assumed to be in PCi_ja form whether or not any PCi_ja
-*       keywords are present since this results in an interpretation of
-*       CDELTia consistent with the original FITS specification.
+*       Matrix elements in the IRAF convention are equivalent to the product
+*       CDi_ja = CDELTia * PCi_ja, but the defaults differ from that of the
+*       PCi_ja matrix.  If one or more CDi_ja keywords are present then all
+*       unspecified CDi_ja default to zero.  If no CDi_ja (or CROTAi) keywords
+*       are present, then the header is assumed to be in PCi_ja form whether
+*       or not any PCi_ja keywords are present since this results in an
+*       interpretation of CDELTia consistent with the original FITS
+*       specification.
 *
 *       While CDi_ja may not formally co-exist with PCi_ja, it may co-exist
 *       with CDELTia and CROTAi which are to be ignored.
 *
 *     - Bit 2: CROTAi is present.
 *
-*       In the AIPS convention, CROTAi may only be
-*       associated with the latitude axis of a celestial axis pair.  It
-*       specifies a rotation in the image plane that is applied AFTER the
-*       CDELTia; any other CROTAi keywords are ignored.
+*       In the AIPS convention, CROTAi may only be associated with the
+*       latitude axis of a celestial axis pair.  It specifies a rotation in
+*       the image plane that is applied AFTER the CDELTia; any other CROTAi
+*       keywords are ignored.
 *
 *       CROTAi may not formally co-exist with PCi_ja.
 *
 *       CROTAi and CDELTia may formally co-exist with CDi_ja but if so are to
 *       be ignored.
 *
-*     CDi_ja and CROTAi keywords, if found, are to be stored in the
-*     wcsprm::cd and wcsprm::crota arrays which are dimensioned similarly to
-*     wcsprm::pc and wcsprm::cdelt.  FITS
-*     header parsers should use the following procedure:
+*     - Bit 3: PCi_ja + CDELTia was derived from CDi_ja by wcspcx().
 *
-*     - Whenever a PCi_ja  keyword is encountered: altlin |= 1;
+*       This bit is set by wcspcx() when it derives PCi_ja and CDELTia from
+*       CDi_ja via an orthonormal decomposition.  In particular, it signals
+*       wcsset() not to replace PCi_ja by a copy of CDi_ja with CDELTia set
+*       to unity.
 *
-*     - Whenever a CDi_ja  keyword is encountered: altlin |= 2;
+*     CDi_ja and CROTAi keywords, if found, are to be stored in the wcsprm::cd
+*     and wcsprm::crota arrays which are dimensioned similarly to wcsprm::pc
+*     and wcsprm::cdelt.  FITS header parsers should use the following
+*     procedure:
+*
+*     - Whenever a PCi_ja keyword is encountered: altlin |= 1;
+*
+*     - Whenever a CDi_ja keyword is encountered: altlin |= 2;
 *
 *     - Whenever a CROTAi keyword is encountered: altlin |= 4;
 *
@@ -1140,8 +1417,9 @@
 *
 *     These alternate specifications of the linear transformation matrix are
 *     translated immediately to PCi_ja by wcsset() and are invisible to the
-*     lower-level WCSLIB routines.  In particular, wcsset() resets
-*     wcsprm::cdelt to unity if CDi_ja is present (and no PCi_ja).
+*     lower-level WCSLIB routines.  In particular, unless bit 3 is also set,
+*     wcsset() resets wcsprm::cdelt to unity if CDi_ja is present (and no
+*     PCi_ja).
 *
 *     If CROTAi are present but none is associated with the latitude axis
 *     (and no PCi_ja or CDi_ja), then wcsset() reverts to a unity PCi_ja
@@ -1547,7 +1825,9 @@
 *   int lat
 *     (Returned) ... index for the latitude coordinate, and ...
 *   int spec
-*     (Returned) ... index for the spectral coordinate in the imgcrd[][] and
+*     (Returned) ... index for the spectral coordinate, and ...
+*   int time
+*     (Returned) ... index for the time coordinate in the imgcrd[][] and
 *     world[][] arrays in the API of wcsp2s(), wcss2p() and wcsmix().
 *
 *     These may also serve as indices into the pixcrd[][] array provided that
@@ -1558,6 +1838,12 @@
 *     is used for quadcube projections where the cube faces are stored on a
 *     separate axis (see wcs.h).
 *
+*   int chksum
+*     (Returned) Checksum of keyvalues provided (see wcsprm::flag).  Used by
+*     wcsenq() to validate the self-consistency of the struct.  Note that
+*     the checksum incorporates addresses and is therefore highly specific to
+*     the instance of the wcsprm struct.
+*
 *   int *types
 *     (Returned) Address of the first element of an array of int containing a
 *     four-digit type code for each axis.
@@ -1567,6 +1853,7 @@
 *       - 1: Stokes coordinate.
 *       - 2: Celestial coordinate (including CUBEFACE).
 *       - 3: Spectral coordinate.
+*       - 4: Time coordinate.
 *
 *     - Second digit (i.e. 100s):
 *       - 0: Linear axis.
@@ -1713,6 +2000,29 @@
 *     (Given, auxiliary) Heliographic latitude (Carrington or Stonyhurst) of
 *     the observer (deg).
 *
+*   double a_radius
+*     Length of the semi-major axis of a triaxial ellipsoid approximating the
+*     shape of a body (e.g. planet) in the solar system (m).
+*
+*   double b_radius
+*     Length of the intermediate axis, normal to the semi-major and semi-minor
+*     axes, of a triaxial ellipsoid approximating the shape of a body (m).
+*
+*   double c_radius
+*     Length of the semi-minor axis, normal to the semi-major axis, of a
+*     triaxial ellipsoid approximating the shape of a body (m).
+*
+*   double blon_obs
+*     Bodycentric longitude of the observer in the coordinate system fixed to
+*     the planet or other solar system body (deg, in range 0 to 360).
+*
+*   double blat_obs
+*     Bodycentric latitude of the observer in the coordinate system fixed to
+*     the planet or other solar system body (deg).
+*
+*   double bdis_obs
+*     Bodycentric distance of the observer (m).
+*
 *
 * Global variable: const char *wcs_errmsg[] - Status return messages
 * ------------------------------------------------------------------
@@ -1732,12 +2042,20 @@ extern "C" {
 #define wtbarr wtbarr_s		// See prologue of wtbarr.h.
 #endif
 
+enum wcsenq_enum {
+  WCSENQ_MEM = 1,		// wcsprm struct memory is managed by WCSLIB.
+  WCSENQ_SET = 2,		// wcsprm struct has been set up.
+  WCSENQ_BYP = 4,		// wcsprm struct is in bypass mode.
+  WCSENQ_CHK = 8,		// wcsprm struct is self-consistent.
+};
+
 #define WCSSUB_LONGITUDE 0x1001
 #define WCSSUB_LATITUDE  0x1002
 #define WCSSUB_CUBEFACE  0x1004
 #define WCSSUB_CELESTIAL 0x1007
 #define WCSSUB_SPECTRAL  0x1008
 #define WCSSUB_STOKES    0x1010
+#define WCSSUB_TIME      0x1020
 
 
 #define WCSCOMPARE_ANCILLARY 0x0001
@@ -1767,7 +2085,8 @@ enum wcs_errmsg_enum {
   WCSERR_NO_SOLUTION     = 11,	// No solution found in the specified
 				// interval.
   WCSERR_BAD_SUBIMAGE    = 12,	// Invalid subimage specification.
-  WCSERR_NON_SEPARABLE   = 13 	// Non-separable subimage coordinate system.
+  WCSERR_NON_SEPARABLE   = 13,	// Non-separable subimage coordinate system.
+  WCSERR_UNSET           = 14 	// wcsprm struct is unset.
 };
 
 
@@ -1798,6 +2117,14 @@ struct auxprm {
   double crln_obs;              // Carrington heliographic lng of observer.
   double hgln_obs;              // Stonyhurst heliographic lng of observer.
   double hglt_obs;              // Heliographic latitude of observer.
+
+  double a_radius;              // Semi-major axis of solar system body.
+  double b_radius;              // Semi-intermediate axis of solar system body.
+  double c_radius;              // Semi-minor axis of solar system body.
+  double blon_obs;              // Bodycentric longitude of observer.
+  double blat_obs;              // Bodycentric latitude of observer.
+  double bdis_obs;              // Bodycentric distance of observer.
+  double dummy[2];              // Reserved for future use.
 };
 
 // Size of the auxprm struct in int units, used by the Fortran wrappers.
@@ -1839,8 +2166,8 @@ struct wcsprm {
   double *cd;			// CDi_ja linear transformation matrix.
   double *crota;		// CROTAi keyvalues for each coord axis.
   int    altlin;		// Alternative representations
-				//   Bit 0: PCi_ja  is present,
-				//   Bit 1: CDi_ja  is present,
+				//   Bit 0: PCi_ja is present,
+				//   Bit 1: CDi_ja is present,
 				//   Bit 2: CROTAi is present.
   int    velref;		// AIPS velocity code, VELREF.
 
@@ -1900,25 +2227,24 @@ struct wcsprm {
   // Information derived from the FITS header keyvalues by wcsset().
   //--------------------------------------------------------------------------
   char   lngtyp[8], lattyp[8];	// Celestial axis types, e.g. RA, DEC.
-  int    lng, lat, spec;	// Longitude, latitude and spectral axis
-				// indices (0-relative).
+  int    lng, lat, spec, time;	// Longitude, latitude, spectral, and time
+				// axis indices (0-relative).
   int    cubeface;		// True if there is a CUBEFACE axis.
+  int    chksum;		// Checksum of keyvalues provided.
   int    *types;		// Coordinate type codes for each axis.
 
   struct linprm lin;		//    Linear transformation parameters.
   struct celprm cel;		// Celestial transformation parameters.
   struct spcprm spc;		//  Spectral transformation parameters.
 
-  //--------------------------------------------------------------------------
-  //             THE REMAINDER OF THE WCSPRM STRUCT IS PRIVATE.
-  //--------------------------------------------------------------------------
-
-  // Error handling, if enabled.
+  // Error messaging, if enabled.
   //--------------------------------------------------------------------------
   struct wcserr *err;
 
-  // Memory management.
   //--------------------------------------------------------------------------
+  // Private - the remainder are for internal use.
+  //--------------------------------------------------------------------------
+				// Memory management.
   int    m_flag, m_naxis;
   double *m_crpix, *m_pc, *m_cdelt, *m_crval;
   char  (*m_cunit)[72], (*m_ctype)[72];
@@ -1956,6 +2282,14 @@ int wcscompare(int cmp, double tol, const struct wcsprm *wcs1,
 
 int wcsfree(struct wcsprm *wcs);
 
+int wcstrim(struct wcsprm *wcs);
+
+int wcssize(const struct wcsprm *wcs, int sizes[2]);
+
+int auxsize(const struct auxprm *aux, int sizes[2]);
+
+int wcsenq(const struct wcsprm *wcs, int enquiry);
+
 int wcsprt(const struct wcsprm *wcs);
 
 int wcsperr(const struct wcsprm *wcs, const char *prefix);
@@ -1972,10 +2306,14 @@ int wcss2p(struct wcsprm *wcs, int ncoord, int nelem, const double world[],
            double phi[], double theta[], double imgcrd[], double pixcrd[],
            int stat[]);
 
-int wcsmix(struct wcsprm *wcs, int mixpix, int mixcel, const double vspan[],
+int wcsmix(struct wcsprm *wcs, int mixpix, int mixcel, const double vspan[2],
            double vstep, int viter, double world[], double phi[],
            double theta[], double imgcrd[], double pixcrd[]);
 
+int wcsccs(struct wcsprm *wcs, double lng2p1, double lat2p1, double lng1p2,
+           const char *clng, const char *clat, const char *radesys,
+           double equinox, const char *alt);
+
 int wcssptr(struct wcsprm *wcs, int *i, char ctype[9]);
 
 const char* wcslib_version(int vers[3]);
